# mandayejs - saml reverse proxy
# Copyright (C) 2015  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

from django.shortcuts import get_object_or_404
from django.contrib.auth.models import User
from django.db import models

from rest_framework import status
from rest_framework.views import APIView
from rest_framework.response import Response

from mandayejs.applications import get_app_settings
from mandayejs.mandaye.models import UserCredentials
from mandayejs.mandaye.utils import exec_phantom, get_login_info, get_idp

from mellon.models import UserSAMLIdentifier


class MandayeAPI(APIView):
    """MandayeJS API
    """

    def get(self, request, *args, **kwargs):
        app_settings = get_app_settings()
        locators = {
            locator.get('name'): ''
            for locator in app_settings.SITE_LOCATORS
        }
        return Response(locators)

    def post(self, request, *args, **kwargs):
        data = request.data

        user, created = User.objects.get_or_create(
            username=data.get('name_id_content'),
            email=data.get('email'),
            first_name=data.get('first_name'),
            last_name=data.get('last_name')
        )

        uuid, created = UserSAMLIdentifier.objects.get_or_create(
            user_id=user.id,
            name_id=data.get('name_id_content'),
            issuer=get_idp()
        )

        cred = UserCredentials(user=user, locators=data.get('locators'))

        login_info = get_login_info(request, cred)

        result = exec_phantom(login_info)
        if result.get('result') != 'ok':
            return Response(status=status.HTTP_401_UNAUTHORIZED)

        cred.save()
        return Response(status=status.HTTP_200_OK)

    def delete(self, request, *args, **kwargs):
        data = request.data
        user = get_object_or_404(User, username=data.get('name_id_content'))
        cred = get_object_or_404(UserCredentials, user=user)
        cred.delete()
        return Response(status=status.HTTP_200_OK)


api = MandayeAPI.as_view()


class MandayeStatsAPI(APIView):

    http_method_names = ['get']

    def get(self, request, *args, **kwargs):
        data = UserCredentials.objects.aggregate(
            users_linked=models.Sum(models.Case(models.When(linked=True, then=1), output_field=models.IntegerField())),
            users_unlinked=models.Sum(models.Case(models.When(linked=False, then=1), output_field=models.IntegerField()))
        )
        return Response(data)


api_stats = MandayeStatsAPI.as_view()
